from . import BaseFile


class MarkdownFile(BaseFile):
    def __init__(self, github_file, repo):
        super().__init__(github_file, repo)

    def _load_objects(self):
        return self.file_contents.split(b"\n")

    def _dump(self, serialize_options={}):
        return b"\n".join(self.file_contents)



from . import BaseFile
import json


class JsonFile(BaseFile):
    def __init__(self, github_file, repo):
        super().__init__(github_file, repo)

    def _load_objects(self):
        return json.loads(self.file_contents)

    def _dump(self, serialize_options={}):
        indent = serialize_options.get("indent", 4)
        return json.dumps(self.objects, indent=indent)



import unittest
from gordian.config import Config

class TestConfig(unittest.TestCase):

    def test_get_data(self):
        test_return = ['testOrg/TestService1', 'testOrg/TestService2']
        config = Config('./tests/fixtures/test_config.yaml')
        data = config.get_data()
        self.assertEqual(data, test_return)



import unittest
import pytest
import os
from gordian.repo import Repo
from github import BadCredentialsException, RateLimitExceededException, UnknownObjectException, GithubException
from unittest.mock import MagicMock, patch, call
from gordian.files import YamlFile
from .utils import Utils

class TestRepo(unittest.TestCase):

    @patch('gordian.repo.Github')
    def setUp(self, mock_git):
        self.mock_git = MagicMock()
        self.mock_repo = MagicMock()
        self.mock_branches = MagicMock()
        self.repo = Repo('test', github=self.mock_git)
        self.repo.files.append(Utils.create_github_content_file())

        self.mock_repo.get_branches.return_value = self.mock_branches
        self.mock_git.get_repo.return_value = self.mock_repo

    @patch('gordian.repo.Github')
    def test_remove_dot_git_from_repo_name(self, mock_git):
        self.mock_git.reset_mock()
        self.repo = Repo('test.git', github=self.mock_git)
        self.mock_git.get_repo.assert_called_once_with('test')

    def test_no_fork(self):
        repo = Repo('test_repo', branch='', github=self.mock_git, fork=False)
        repo._target_repo.create_fork.assert_not_called()
        self.assertEqual(repo._source_repo, repo._target_repo)

    def test_fork(self):
        repo = Repo('test_repo', branch='', github=self.mock_git, fork=True)
        repo._target_repo.create_fork.assert_called_once()
        self.assertNotEqual(repo._source_repo, repo._target_repo)

    def test_make_branch_fork(self):
        repo = Repo('test_repo', branch='', github=self.mock_git, fork=True)
        repo.branch_exists = False
        mock_branch = MagicMock()
        self.mock_repo.get_branch.return_value = mock_branch
        mock_branch.commit.sha = "5e69ff00a3be0a76b13356c6ff42af79ff469ef3"
        repo._make_branch()
        self.assertTrue(repo.branch_exists)
        repo._source_repo.get_branch.assert_called_once_with('master')
        repo._source_repo.create_git_ref.assert_called_once()

    def test_make_branch_no_fork(self):
        repo = Repo('test_repo', branch='', github=self.mock_git, fork=False)
        repo.branch_exists = False
        mock_branch = MagicMock()
        self.mock_repo.get_branch.return_value = mock_branch
        mock_branch.commit.sha = "5e69ff00a3be0a76b13356c6ff42af79ff469ef3"
        repo._make_branch()
        self.assertTrue(repo.branch_exists)
        repo._source_repo.get_branch.assert_called_once_with('master')
        repo._source_repo.create_git_ref.assert_called_once()

    def test_default_github_url(self):
        self.assertEqual(self.repo.github_api_url, 'https://api.github.com')

    def test_override_github_url(self):
        repo = Repo('test', github_api_url='https://test.github.com', github=self.mock_git)
        self.assertEqual(repo.github_api_url, 'https://test.github.com')

    def test_get_object_does_not_exist(self):
        with pytest.raises(FileNotFoundError):
            self.repo.get_objects('test')

    def test_get_existing_object(self):
        contents = self.repo.get_objects('/content.yaml')
        assert(isinstance(contents, YamlFile))

    def test_new_files_object(self):
        self.assertEqual(len(self.repo.files), 1)
        repo_two = Repo('test_two', github_api_url='https://test.github.com', github=MagicMock())
        self.assertEqual(len(repo_two.files), 0)

    def test_get_files(self):
        self.repo._set_target_branch('target')
        self.repo.files = []
        self.repo._source_repo = MagicMock()
        repository_file = MagicMock(path='afile.txt', type='not_dir')
        self.repo._source_repo.get_contents.side_effect = [[MagicMock(path='directory', type='dir')],[repository_file]]
        self.repo.get_files()
        self.repo._source_repo.get_contents.assert_has_calls([call('', 'target'), call('directory', 'target')])
        self.assertEqual(self.repo.files, [repository_file])

    def test_get_files_bad_credentials_exception(self):
        self.repo._set_target_branch('target')
        self.repo.files = []
        self.repo._source_repo = MagicMock()
        self.repo._source_repo.get_contents.side_effect = BadCredentialsException('Bad Credentials')
        with pytest.raises(BadCredentialsException):
            self.repo.get_files()

    def test_get_files_rate_limit_exception(self):
        self.repo._set_target_branch('target')
        self.repo.files = []
        self.repo._source_repo = MagicMock()
        self.repo._source_repo.get_contents.side_effect = RateLimitExceededException('Rate Limit Exceeded')
        with pytest.raises(RateLimitExceededException):
            self.repo.get_files()

    def test_get_files_unknown_object_exception(self):
        self.repo._set_target_branch('target')
        self.repo.files = []
        self.repo._source_repo = MagicMock()
        self.repo._source_repo.get_contents.side_effect = UnknownObjectException('Unknown Object')
        with pytest.raises(UnknownObjectException):
            self.repo.get_files()

    def test_get_files_generic_github_exception(self):
        self.repo._set_target_branch('target')
        self.repo.files = []
        self.repo._source_repo = MagicMock()
        self.repo._source_repo.get_contents.side_effect = GithubException('Generic Exception')
        with pytest.raises(GithubException):
            self.repo.get_files()

    def test_create_branch_generic_github_exception_fall_through(self):
        self.repo._set_target_branch('target')
        self.repo.files = []
        self.repo._source_repo = MagicMock()
        self.repo._source_repo.create_git_ref.side_effect = GithubException('Branch already exists')

        self.repo._make_branch()
        self.repo._source_repo.create_git_ref.assert_called_once()
        assert self.repo.branch_exists

    def test_set_target_branch(self):
        self.repo._set_target_branch('master')
        self.assertEqual(self.repo.source_branch, 'refs/heads/master')

    def test_set_target_branch_source_branch(self):
        self.repo._set_target_branch('master', 'something')
        self.assertEqual(self.repo.source_branch, 'refs/heads/something')

    def test_set_target_branch_reset_file_cache(self):
        self.repo._set_target_branch('master')
        cached_files = ['cached_file', 'cached_file', 'cached_file']
        self.repo.files = cached_files
        self.assertEqual(self.repo.files, cached_files)

        self.repo._set_target_branch('Something different')
        self.assertEqual(self.repo.files, [])
        self.assertEqual(self.repo.target_branch, 'Something different')
        self.assertEqual(self.repo.target_ref, 'refs/heads/Something different')

    def test_create_pr(self):
        repo = Repo('test_repo', branch='', github=self.mock_git)
        repo._target_repo = MagicMock()
        repo._source_repo = MagicMock()
        repo._source_repo.owner.login = 'someone'
        repo.branch_name = 'branch'
        pr = repo.create_pr('test', '', 'target_branch', ['test'])
        repo._target_repo.create_pull.assert_called_once_with(title='test', body='', base='target_branch', head='someone:branch', draft=False)
        pr.set_labels.assert_called_once_with('test')
        repo._source_repo.create_pull.assert_not_called()

    def test_create_pr_draft(self):
        repo = Repo('test_repo', branch='', github=self.mock_git)
        repo._target_repo = MagicMock()
        repo._source_repo = MagicMock()
        repo._source_repo.owner.login = 'someone'
        repo.branch_name = 'branch'
        pr = repo.create_pr('test', '', 'target_branch', ['test'], draft=True)
        repo._target_repo.create_pull.assert_called_once_with(title='test', body='', base='target_branch', head='someone:branch', draft=True)
        pr.set_labels.assert_called_once_with('test')
        repo._source_repo.create_pull.assert_not_called()

    def test_create_pr_named_arguments(self):
        repo = Repo('test_repo', branch='', github=self.mock_git)
        repo._target_repo = MagicMock()
        repo._source_repo = MagicMock()
        repo._source_repo.owner.login = 'someone'
        repo.branch_name = 'branch'
        pr = repo.create_pr(pr_message='test', pr_body='', target_branch='target_branch', labels=['test'], draft=True)
        repo._target_repo.create_pull.assert_called_once_with(title='test', body='', base='target_branch', head='someone:branch', draft=True)
        pr.set_labels.assert_called_once_with('test')
        repo._source_repo.create_pull.assert_not_called()

    def test_create_pr_no_labels(self):
        repo = Repo('test_repo', branch='', github=self.mock_git)
        repo._target_repo = MagicMock()
        repo._source_repo = MagicMock()
        repo._source_repo.owner.login = 'someone'
        repo.branch_name = 'branch'
        pr = repo.create_pr('test', '', 'target_branch')
        repo._target_repo.create_pull.assert_called_once_with(title='test', body='', base='target_branch', head='someone:branch', draft=False)
        pr.set_labels.assert_not_called()
        repo._source_repo.create_pull.assert_not_called()

    def test_get_new_version_major(self):
        version_file = MagicMock()
        version_file.decoded_content = '1.2.3'.encode('utf-8')
        self.repo.version_file = version_file
        self.repo.semver_label = 'major'
        self.repo._get_new_version()
        self.assertEqual(self.repo.new_version, '2.0.0')

    def test_get_new_version_minor(self):
        version_file = MagicMock()
        version_file.decoded_content = '1.2.3'.encode('utf-8')
        self.repo.version_file = version_file
        self.repo.semver_label = 'minor'
        self.repo._get_new_version()
        self.assertEqual(self.repo.new_version, '1.3.0')

    def test_get_new_version_patch(self):
        version_file = MagicMock()
        version_file.decoded_content = '1.2.3'.encode('utf-8')
        self.repo.version_file = version_file
        self.repo.semver_label = 'patch'
        self.repo._get_new_version()
        self.assertEqual(self.repo.new_version, '1.2.4')

    @patch('gordian.repo.Github')
    def test_init_with_passed_token(self, mock_git):
        Repo('test_repo', token='abcdef')
        args = {'login_or_token': 'abcdef', 'base_url': 'https://api.github.com', 'lazy':False, 'pool_size':None}
        mock_git.assert_called_with(**args)

    @patch.dict(os.environ, {'GIT_TOKEN': '12345'})
    @patch('gordian.repo.Github')
    def test_init_with_token_from_env(self, mock_git):
        Repo('test_repo')
        args = {'login_or_token': '12345', 'base_url': 'https://api.github.com', 'lazy':False, 'pool_size':None}

        mock_git.assert_called_with(**args)

    @patch.dict(os.environ, {'GIT_USERNAME': 'test-user', 'GIT_PASSWORD': 'test-pass'})
    @patch('gordian.repo.Github')
    def test_init_with_user_pass_env(self, mock_git):
        Repo('test_repo')
        args = {'login_or_token':'test-user', 'password':'test-pass', 'base_url': 'https://api.github.com', 'lazy':False, 'pool_size':None}

        mock_git.assert_called_with(**args)

    @patch('gordian.repo.Github')
    def test_create_file(self, mock_git):
        repo = Repo('test_repo', github=mock_git)
        repo.create_file('test', 'test', 'test file create')

        repo._source_repo.create_file.assert_called_once()
        self.assertTrue(repo.dirty)

    @patch('gordian.repo.Github')
    def test_delete_file(self, mock_git):
        mock_file = MagicMock()
        repo = Repo('test_repo', github=mock_git)
        repo.delete_file(mock_file, 'test file delete')

        repo._source_repo.delete_file.assert_called_once()
        self.assertTrue(repo.dirty)

    def test_get_files_with_path(self):
        self.repo._set_target_branch('target')
        self.repo.files = []
        self.repo._source_repo = MagicMock()
        repository_file = MagicMock(path='test/afile.txt', type='not_dir')
        self.repo._source_repo.get_contents.side_effect = [[MagicMock(path='directory', type='dir')],[repository_file]]
        self.repo.get_files('test')
        self.repo._source_repo.get_contents.assert_has_calls([call('test', 'target'), call('directory', 'target')])
        self.assertEqual(self.repo.files, [repository_file])

    def test__get_github_client(self):
        repo = Repo('test_repo', branch='', github=self.mock_git)

        self.assertIsNotNone(repo.get_github_client())
        self.assertEqual(repo.get_github_client(), self.mock_git)

    def test_get_repo_contents_timeout_error(self):
        self.repo._set_target_branch('target')
        self.repo.files = []
        self.repo._source_repo = MagicMock()
        self.repo._source_repo.get_contents.side_effect = TimeoutError('Read Timeout')
        with pytest.raises(Exception) as context:
            self.repo._get_repo_contents(path='test/afile.txt')
        assert "Read Timeout" in str(context.value)
        self.repo._source_repo.get_contents.assert_has_calls([call('test/afile.txt', 'target'), call('test/afile.txt', 'target'), call('test/afile.txt', 'target')])


    def test_get_repo_contents(self):
        self.repo._set_target_branch('target')
        self.repo.files = []
        self.repo._source_repo = MagicMock()
        repository_file = MagicMock(path='test/afile.txt', type='not_dir')
        self.repo._source_repo.get_contents.side_effect = [repository_file]
        self.repo._get_repo_contents(path='test/afile.txt')
        self.repo._source_repo.get_contents.assert_has_calls([call('test/afile.txt', 'target')])



import unittest
from unittest.mock import MagicMock
from gordian.repo import Repo
from gordian.files import YamlFile
from .utils import Utils


class TestBaseFile(unittest.TestCase):
    def setUp(self):
        self.github_file = Utils.create_github_content_file()
        self.mock_git = MagicMock()
        self.repo = Repo("test", github=self.mock_git)
        self.base_file = YamlFile(self.github_file, self.repo)

    def test_iterable(self):
        assert iter(self.base_file)


class TestYamlFile(unittest.TestCase):
    def setUp(self):
        self.github_file = Utils.create_github_content_file()
        self.mock_git = MagicMock()
        self.repo = Repo("test", github=self.mock_git)
        self.base_file = YamlFile(self.github_file, self.mock_git)

    def test_save_options(self):
        self.base_file.save("Test message", False, {"explicit_start": False})
        self.mock_git.update_file.assert_called_once_with(
            self.github_file, "test:\n  foo: bar\n  iam: blah\n", "Test message", False
        )

    def test_save_default_options(self):
        self.base_file.save("Test message", False)
        self.mock_git.update_file.assert_called_once_with(
            self.github_file,
            "---\ntest:\n  foo: bar\n  iam: blah\n",
            "Test message",
            False,
        )



import logging

logger = logging.getLogger(__name__)


class BaseFile:
    def __init__(self, github_file, repo, start=0):
        self.github_file = github_file
        self.repo = repo
        self.num = start
        self.file_contents = github_file.decoded_content
        self.objects = self._load_objects()

    def __iter__(self):
        return self

    def __next__(self):
        if self.num == len(self.objects):
            raise StopIteration

        num = self.num
        self.num += 1
        return self.objects[num]

    def save(self, message, dry_run, serialize_options={}):
        self.repo.update_file(
            self.github_file, self._dump(serialize_options), message, dry_run
        )

    def _load_objects(self):
        raise NotImplementedError

    def _dump(self, serialize_options={}):
        raise NotImplementedError



import unittest
from gordian.config import Config
from gordian.gordian import apply_transformations
from unittest.mock import MagicMock, patch, call, Mock, mock_open, ANY


class TestGordian(unittest.TestCase):

    class Args(object):
        def __init__(self, config_file='./tests/fixtures/test_config.yaml', dry_run = False, lazy = False, pool_size = None, draft = False):
            self.config_file = config_file
            self.major = False
            self.minor = False
            self.patch = False
            self.dry_run = dry_run
            self.pr_message = 'test'
            self.branch = 'test'
            self.github_api = None
            self.semver_label = None
            self.target_branch = 'master'
            self.pr_labels = ['test']
            self.description = ''
            self.description_file = None
            self.fork = False
            self.lazy = lazy
            self.pool_size = pool_size
            self.draft = draft

    def test_apply_transformations_without_changes(self):
        with patch('gordian.gordian.Repo') as RepoMock, patch('gordian.transformations.Transformation') as TransformationMockClass:
            instance = RepoMock.return_value
            instance.dirty = False
            apply_transformations(TestGordian.Args(), [TransformationMockClass])
            RepoMock.assert_has_calls([
                call('testOrg/TestService1', github_api_url=None, branch='test', semver_label=None, target_branch='master', fork=False, lazy=False, pool_size=None),
                call('testOrg/TestService2', github_api_url=None, branch='test', semver_label=None, target_branch='master', fork=False, lazy=False, pool_size=None)
            ])

    def test_apply_transformations_with_changes(self):
        with patch('gordian.gordian.Repo') as RepoMock, patch('gordian.transformations.Transformation') as TransformationMockClass:
            instance = RepoMock.return_value
            instance.dirty = True
            apply_transformations(TestGordian.Args(), [TransformationMockClass])
            RepoMock.assert_has_calls([call().bump_version(False), call().bump_version(False)], any_order=True)
            RepoMock.assert_has_calls([
                call().create_pr(pr_message='test', pr_body='', target_branch='master', labels=ANY, draft=False),
                call().create_pr(pr_message='test', pr_body='', target_branch='master', labels=ANY, draft=False)], any_order=True)

    def test_apply_transformations_with_changes_dry_run(self):
        with patch('gordian.gordian.Repo') as RepoMock, patch('gordian.transformations.Transformation') as TransformationMockClass:
            instance = RepoMock.return_value
            instance.dirty = True
            apply_transformations(TestGordian.Args(dry_run=True), [TransformationMockClass])
            RepoMock.assert_has_calls([call().bump_version(True), call().bump_version(True)], any_order=True)
            self.assertNotIn(call().repo.create_pr(pr_message='test', pr_body='', target_branch='master', labels=ANY, draft=False), RepoMock.mock_calls)

    def test_apply_transformations_with_changes_draft(self):
        with patch('gordian.gordian.Repo') as RepoMock, patch('gordian.transformations.Transformation') as TransformationMockClass:
            instance = RepoMock.return_value
            instance.dirty = True
            apply_transformations(TestGordian.Args(draft=True), [TransformationMockClass])
            RepoMock.assert_has_calls([call().bump_version(False), call().bump_version(False)], any_order=True)
            RepoMock.assert_has_calls([
                call().create_pr(pr_message='test', pr_body='', target_branch='master', labels=ANY, draft=True),
                call().create_pr(pr_message='test', pr_body='', target_branch='master', labels=ANY, draft=True)], any_order=True)

    def test_apply_transformations_with_changes_lazy(self):
        with patch('gordian.gordian.Repo') as RepoMock, patch('gordian.transformations.Transformation') as TransformationMockClass:
            instance = RepoMock.return_value
            instance.dirty = True
            apply_transformations(TestGordian.Args(lazy=True), [TransformationMockClass])
            RepoMock.assert_has_calls([call().bump_version(False), call().bump_version(False)], any_order=True)
            RepoMock.assert_has_calls([
                call('testOrg/TestService1', github_api_url=None, branch='test', semver_label=None, target_branch='master', fork=False, lazy=True, pool_size=None),
                call().create_pr(pr_message='test', pr_body='', target_branch='master', labels=ANY, draft=False),
                call().create_pr(pr_message='test', pr_body='', target_branch='master', labels=ANY, draft=False)], any_order=True)

    def test_apply_transformations_with_changes_pool_size(self):
        with patch('gordian.gordian.Repo') as RepoMock, patch('gordian.transformations.Transformation') as TransformationMockClass:
            instance = RepoMock.return_value
            instance.dirty = True
            apply_transformations(TestGordian.Args(pool_size=10), [TransformationMockClass])
            RepoMock.assert_has_calls([call().bump_version(False), call().bump_version(False)], any_order=True)
            RepoMock.assert_has_calls([
                call('testOrg/TestService1', github_api_url=None, branch='test', semver_label=None, target_branch='master', fork=False, lazy=False, pool_size=10),
                call().create_pr(pr_message='test', pr_body='', target_branch='master', labels=ANY, draft=False),
                call().create_pr(pr_message='test', pr_body='', target_branch='master', labels=ANY, draft=False)], any_order=True)

    def test_apply_transformations_with_changes_and_callback(self):
        with patch('gordian.gordian.Repo') as RepoMock, patch('gordian.transformations.Transformation') as TransformationMockClass:
            instance = RepoMock.return_value
            instance.dirty = True
            callback_mock = MagicMock()
            args = TestGordian.Args()
            args.target_branch = 'target_branch'
            apply_transformations(args, [TransformationMockClass], callback_mock)
            pull_request = RepoMock.return_value.create_pr.return_value
            RepoMock.assert_has_calls([call().bump_version(False), call().bump_version(False)], any_order=True)
            RepoMock.assert_has_calls([
                call().create_pr(pr_message='test', pr_body='', target_branch='target_branch', labels=ANY, draft=False),
                call().create_pr(pr_message='test', pr_body='', target_branch='target_branch', labels=ANY, draft=False)],
                any_order=True
            )
            callback_mock.assert_has_calls([
                call('testOrg/TestService1', pull_request),
                call('testOrg/TestService2', pull_request)]
            )

    def test_apply_transformations_with_changes_default_labels(self):
        with patch('gordian.gordian.Repo') as RepoMock, patch('gordian.transformations.Transformation', ) as TransformationMockClass:
            instance = RepoMock.return_value
            instance.dirty = True
            gordian_args = TestGordian.Args()
            gordian_args.pr_labels = []
            apply_transformations(gordian_args, [TransformationMockClass])
            RepoMock.assert_has_calls([call().bump_version(False), call().bump_version(False)], any_order=True)
            RepoMock.assert_has_calls([
                call().create_pr(pr_message='test', pr_body='', target_branch='master', labels=ANY, draft=False),
                call().create_pr(pr_message='test', pr_body='', target_branch='master', labels=ANY, draft=False)], any_order=True)
            self.assertNotIn(call()._repo.create_pull().set_labels(ANY), RepoMock.mock_calls)

    def test_apply_transformations_with_changes_custom_description(self):
        with patch('gordian.gordian.Repo') as RepoMock, patch('gordian.transformations.Transformation', ) as TransformationMockClass:
            instance = RepoMock.return_value
            instance.dirty = True
            gordian_args = TestGordian.Args()
            description = 'Custom file for pr description\n'
            gordian_args.description_file = './tests/fixtures/pr_description'
            apply_transformations(gordian_args, [TransformationMockClass])
            RepoMock.assert_has_calls([call().bump_version(False), call().bump_version(False)], any_order=True)
            RepoMock.assert_has_calls([
                call().create_pr(pr_message='test', pr_body=description, target_branch='master', labels=ANY, draft=False),
                call().create_pr(pr_message='test', pr_body=description, target_branch='master', labels=ANY, draft=False)], any_order=True)



import unittest
from gordian.gordian import create_parser

class TestParser(unittest.TestCase):

    def test_no_args(self):
        with self.assertRaises(SystemExit):
            create_parser([])

    def test_just_config(self):
        with self.assertRaises(SystemExit):
            create_parser(['-c', './gordian/config/ex_config.yaml', '--pr', ''])

    def test_just_search(self):
        with self.assertRaises(SystemExit):
            create_parser(['-s', 'test'])

    def test_just_gordian(self):
        with self.assertRaises(SystemExit):
            create_parser(['-r', 'test'])

    def test_config_and_search(self):
        with self.assertRaises(SystemExit):
            create_parser(['-c', './gordian/config/ex_config.yaml', '-s', 'test'])

    def test_config_and_repace(self):
        with self.assertRaises(SystemExit):
            create_parser(['-c', './gordian/config/ex_config.yaml', '-r', 'test'])

    def test_search_and_gordian(self):
        args = create_parser(['-s', 'hello', '-r', 'goodbye', '--pr', 'test'])
        self.assertEqual(args.config_file, 'config.yaml')
        self.assertEqual(args.search, ['hello'])
        self.assertEqual(args.replace, ['goodbye'])

    def test_all_args(self):
        args = create_parser(['-c', './gordian/config/ex_config.yaml', '-s', 'hello', '-r', 'goodbye', '--pr', 'test'])
        self.assertEqual(args.config_file, './gordian/config/ex_config.yaml')
        self.assertEqual(args.search, ['hello'])
        self.assertEqual(args.replace, ['goodbye'])



from . import BaseFile
import yaml


class YamlFile(BaseFile):
    def __init__(self, github_file, repo):
        super().__init__(github_file, repo)
        yaml.add_representer(type(None), represent_none)

    def _load_objects(self):
        return list(yaml.safe_load_all(self.file_contents))

    def _dump(self, serialize_options={}):
        default_flow_style = serialize_options.get("default_flow_style", False)
        explicit_start = serialize_options.get("explicit_start", True)
        return yaml.dump_all(
            self.objects,
            default_flow_style=default_flow_style,
            explicit_start=explicit_start,
        )


def represent_none(self, _):
    # Disable dumping 'null' string for null values
    # Taken from here: https://stackoverflow.com/a/41786451
    return self.represent_scalar("tag:yaml.org,2002:null", "")



import unittest

from gordian.files.plaintext_file import PlainTextFile
from gordian.repo import Repo
from gordian.transformations import SearchAndReplace, PlainTextUpdater
from unittest.mock import MagicMock, patch

from tests.utils import Utils


class TestSearchAndReplaceTransformation(unittest.TestCase):

    class Args(object):
        def __init__(self, dry_run=False, search=['iam'], replace=['hello'], file='content.txt'):
            self.dry_run = dry_run
            self.search = search
            self.replace = replace
            self.file = file

    @patch('gordian.repo.Github')
    def setUp(self, mock_git):
        self.mock_repo = MagicMock()
        self.mock_branches = MagicMock()
        self.mock_repo.get_branches.return_value = self.mock_branches
        mock_git.get_repo.return_value = self.mock_repo
        self.mock_repo.get_contents.return_value = []
        self.instance = Repo('test_repo', branch='', github=mock_git, files=[])
        self.instance.branch_exists = False
        f = open('./tests/fixtures/content.yaml', 'r')
        contents = f.read()
        mock_file = MagicMock()
        mock_file.decoded_content = contents.encode()
        self.instance.files.append(mock_file)
        self.sandr = SearchAndReplace(TestSearchAndReplaceTransformation.Args(), self.instance)

    def test_both_false_update_files(self):
        assert(self.sandr.run())
        self.instance._source_repo.create_git_ref.assert_called_once()
        self.instance._source_repo.update_file.assert_called_once()

    def test_is_word_found(self):
        f = open('./tests/fixtures/content.yaml', 'r')
        contents = f.read()
        self.assertTrue(self.sandr.is_word_found(contents, 'iam'))
        self.assertFalse(self.sandr.is_word_found(contents, 'hello'))

    def test_pr_false_update_files(self):
        source_repo = MagicMock()
        self.instance._source_repo = source_repo
        self.instance.branch_exists = True
        assert(self.sandr.run())
        source_repo.update_file.assert_called_once()

    def test_update_files_none(self):
        self.sandr.changesets = (('hello', 'iam'),)
        self.sandr.run()
        self.mock_repo.update_file.assert_not_called()
        self.mock_repo.create_pull.assert_not_called()

    def test_update_files_empty(self):
        self.instance.files = []
        self.sandr.run()
        self.mock_repo.update_file.assert_not_called()
        self.mock_repo.create_pull.assert_not_called()

    def test_PlainTextUpdater(self):
        self.file = Utils.create_github_content_file(file='content.txt')
        self.ptf = PlainTextFile(self.file, self.instance)
        self.instance.files = [self.ptf.github_file]
        self.ptu = PlainTextUpdater(TestSearchAndReplaceTransformation.Args(), self.instance)
        self.assertIs(self.ptu.run(), True)



import unittest
from unittest.mock import MagicMock, patch
from gordian.repo import Repo
from gordian.files import ChangelogFile
from .utils import Utils
from datetime import datetime

class TestBaseFile(unittest.TestCase):

    def setUp(self):
        self.github_file = Utils.create_github_content_file(file='changelog_no_footer.md')
        self.mock_git = MagicMock()
        self.repo = Repo('test', github=self.mock_git)
        self.repo.new_version = '1.2.0'
        self.changelog = ChangelogFile(self.github_file, self.repo)

    def test_iterable(self):
        assert(iter(self.changelog))

    def test_assert_added(self):
        self.changelog.added('test')
        assert(len(self.changelog._added) == 1)
        assert(self.changelog._added[0] == ('test', None))

    def test_assert_added_with_ticket(self):
        self.changelog.added('test', 'something-1234')
        assert(len(self.changelog._added) == 1)
        assert(self.changelog._added[0] == ('test', 'something-1234'))

    def test_assert_updated(self):
        self.changelog.updated('test')
        assert(len(self.changelog._updated) == 1)
        assert(self.changelog._updated[0] == ('test', None))

    def test_assert_removed(self):
        self.changelog.removed('test')
        assert(len(self.changelog._removed) == 1)
        assert(self.changelog._removed[0] == ('test', None))

    def test_save_changelog(self):
        self.changelog.added('test')
        self.changelog.save('save file', False)

    def test_changelog_format_no_footer(self):
        changelog = '''# Changelog

## [1.2.0] - 2020-06-02
### Added
- something
### Removed
- something else [ticket-1234]

## [1.1.0] - 2020-02-15
### Added
- Something new JIRA-10000
- Something else SRE-11000

## [1.0.0] - 2020-02-14
### Changed
- Foobar SRE-9999
### Removed
- SRE-8454 Removed a feature

'''
        self.changelog.added('something')
        self.changelog.removed('something else', 'ticket-1234')
        with patch('gordian.files.changelog_file.ChangelogFile._format_date', return_value=datetime(2020, 6, 2).strftime('%Y-%m-%d')):
            assert(self.changelog._dump() == changelog)

    def test_changelog_format_with_footer(self):
        self.github_file = Utils.create_github_content_file(file='changelog_with_footer.md')
        self.changelog = ChangelogFile(self.github_file, self.repo)
        changelog = '''# Changelog

## [1.2.0] - 2020-06-02
### Added
- something
### Removed
- something else [ticket-1234]

## [1.1.0] - 2020-02-15
### Added
- Something new JIRA-10000
- Something else SRE-11000

## [1.0.0] - 2020-02-14
### Changed
- Foobar SRE-9999
### Removed
- SRE-8454 Removed a feature

this is a footer'''
        self.changelog.added('something')
        self.changelog.removed('something else', 'ticket-1234')
        with patch('gordian.files.changelog_file.ChangelogFile._format_date', return_value=datetime(2020, 6, 2).strftime('%Y-%m-%d')):
            assert(self.changelog._dump() == changelog)



import setuptools
from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setup_reqs = ["pytest", "pytest-cov", "pytest-runner", "flake8"]
setuptools.setup(
    name="gordian",
    version="3.8.2",
    author="Intuit",
    author_email="cg-sre@intuit.com",
    description="A tool to search and replace files in a Git repo",
    long_description=long_description,
    long_description_content_type='text/markdown',
    url="https://github.com/argoproj-labs/gordian",
    install_requires=["pygithub<2.6.1", "pyyaml", "jsonpatch", "deepdiff", "retry"],
    setup_requires=setup_reqs,
    extras_require={"test": setup_reqs},
    tests_require=setup_reqs,
    packages=["gordian", "gordian.files"],
    entry_points={
        "console_scripts": [
            "gordian = gordian.gordian:main",
            "pulpo = gordian.pulpo:main",
        ],
    },
)
